using System;
using System.Runtime.InteropServices;

namespace com.ibm.gre
{

	/// <summary>
	/// This class encapsulates login and creation methods for the
	/// IRMWebServices proxy classes.
	/// To use this class create a new ClientSession object specifying
	/// the serverNamePort value with the name of the IBM DB2 WebServices server
	/// followed by the port number. For example "localhost:9080".
	/// 
	/// The class contains methods for getting a list of available data sources,
	/// logging in, and getting the various Controller classes.
	/// 
	/// Call the getDataSourceList method first to get an XML formatted string
	/// containing the list of available data sources. For information on the
	/// return value for this method or other Controller methods see the
	/// IBM DB2 Records Manager API Reference.
	/// 
	/// Once you have determined the data source you wish to use
	/// Call the login method of your choice to initialize the ClientController
	/// so that you may then get the remaining Controller objects.
	/// 
	/// This class encapsultes all the operations that are necessary to
	/// create and use the IRMWebServices proxy classes.
	/// 
	/// Follow the code steps in the Form1_Load and doLogin methods in the Form1 class
	/// to see how easy it is to instantiate, get a data source and
	/// login to the IBM DB2 Records Manager WebService.
	/// 
	/// After you have created a ClientSession object and logged in it is a simple matter to
	/// call any of the Controller methods as demonstrated in the
	/// button1_Click method in the Form1 class. 
	/// </summary>
	
	public class ClientSession
	{
		private string m_serverNamePort;

		private string m_username;
		private string m_password;
		private string m_datasource = null;
		private string m_host;
		private string m_languageCode;

		private bool isHostLogin = false;

		private string m_extUserId;
		private string m_hostName;
		private string m_hostUserName;
		private string m_hostPassword;
		private string m_groupListXML;

		private LoginManagerEJBService m_LoginManager = null;

		private ClientControllerEJBService m_ClientController = null;

		private BatchControllerEJBService m_BatchController = null;
		private ComponentDesignControllerEJBService m_ComponentDesignController = null;
		private ConfigurationControllerEJBService m_ConfigurationController = null;
		private DefinitionControllerEJBService m_DefinitionController = null;
		private FilePlanComponentContentControllerEJBService m_FilePlanComponentContentController = null;
		private FilePlanComponentControllerEJBService m_FilePlanComponentController = null;
		private FilePlanDesignControllerEJBService m_FilePlanDesignController = null;
		private ImportExportControllerEJBService m_ImportExportController = null;
		private LifeCycleDesignControllerEJBService m_LifeCycleDesignController = null;
		private NavigationControllerEJBService m_NavigationController = null;
		private QueryControllerEJBService m_QueryController = null;
		private RecordsOperationsControllerEJBService m_RecordsOperationsController = null;
		private SecurityControllerEJBService m_SecurityController = null;
		private SecurityManagementControllerEJBService m_SecurityManagementController = null;
		private TemplateControllerEJBService m_TemplateController = null;
		private ToolsControllerEJBService m_ToolsController = null;
		private UserControllerEJBService m_UserController = null;

		private void invalidate()
		{
			m_LoginManager = null;
			m_ClientController = null;
			m_BatchController = null;
			m_ComponentDesignController = null;
			m_ConfigurationController = null;
			m_DefinitionController = null;
			m_FilePlanComponentContentController = null;
			m_FilePlanComponentController = null;
			m_FilePlanDesignController = null;
			m_ImportExportController = null;
			m_LifeCycleDesignController = null;
			m_NavigationController = null;
			m_QueryController = null;
			m_RecordsOperationsController = null;
			m_SecurityController = null;
			m_SecurityManagementController = null;
			m_TemplateController = null;
			m_ToolsController = null;
			m_UserController = null;
		}

		public ClientSession(string serverNamePort)
		{
			m_serverNamePort = serverNamePort;
		}
		
		private LoginManagerEJBService getLoginManager()
		{
			if(m_LoginManager == null)
			{
				String url = "http://" + m_serverNamePort + "/IRMWebServices/services/LoginManagerEJB";
				m_LoginManager = new LoginManagerEJBService(url);
			}
			return m_LoginManager;
		}

		private void initClientController(String serviceId)
		{
			String url = "http://" + m_serverNamePort + "/IRMWebServices/services/ClientControllerEJB";
			m_ClientController = new ClientControllerEJBService(serviceId,url);
		}
		
		public string getCurrentDataSource()
		{
			return m_datasource;
		}

		public string getDataSourceList()
		{
			return getLoginManager().getDataSourceList();
		}

		public string getEngineVersion()
		{
			return getLoginManager().getEngineVersion();
		}

		public void logout()
		{
			if(m_ClientController != null)
			{
				try
				{
					m_ClientController.logout();
				}//If Session Bean timed out or some such problem
				catch(System.Web.Services.Protocols.SoapException s){System.Console.Error.WriteLine(s.Message);}
				catch(System.Net.WebException we){System.Console.Error.WriteLine(we.Message);}
			}
			this.invalidate();
		}

		public void reLogin()
		{
			this.logout();
			if(isHostLogin)
			{
				hostLogin(m_extUserId,m_hostName,m_hostUserName,m_hostPassword,m_datasource,m_groupListXML,m_languageCode);
			}
			else
			{
				login(m_username,m_password,m_datasource,m_host,m_languageCode);
			}
		}
		
		public string getSerializedClientController()
		{
			return getLoginManager().integrationLogin(m_username, m_password, m_datasource, m_host, m_languageCode);
		}

		public void login(string username, string password, string datasource, string host, string languageCode)
		{
			initClientController(getLoginManager().integrationLogin(username, password, datasource, host, languageCode));
			m_username = username;
			m_password = password;
			m_datasource = datasource;
			m_host = host;
			m_languageCode = languageCode;
		}

		public void hostLogin(string extUserId, string hostName, string hostUserName, string hostPassword, string database, string groupListXML, string languageCode)
		{
			initClientController(getLoginManager().integrationHostLogin(extUserId, hostName, hostUserName, hostPassword, database, groupListXML, languageCode));
			isHostLogin = true;
			m_extUserId = extUserId;
			m_hostName = hostName;
			m_hostUserName = hostUserName;
			m_hostPassword = hostPassword;
			m_datasource = database;
			m_groupListXML = groupListXML;
			m_languageCode = languageCode;
		}

		public void loginAndUpdatePassword(string username, string oldPassword, string newPassword, string datasource, string host, string languageCode)
		{
			initClientController(getLoginManager().integrationLoginAndUpdatePassword(username, oldPassword, newPassword, datasource, host, languageCode));
			m_username = username;
			m_password = newPassword;
			m_datasource = datasource;
			m_host = host;
			m_languageCode = languageCode;
		}

		public BatchControllerEJBService getBatchController()
		{
			if(m_BatchController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/BatchControllerEJB";
				string serviceId = m_ClientController.getBatchControllerAsString();
				m_BatchController = new BatchControllerEJBService(serviceId,url);
			}
			return m_BatchController;
		}

		public ComponentDesignControllerEJBService getComponentDesignController()
		{
			if(m_ComponentDesignController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/ComponentDesignControllerEJB";
				string serviceId = m_ClientController.getComponentDesignControllerAsString();
				m_ComponentDesignController = new ComponentDesignControllerEJBService(serviceId,url);
			}
			return m_ComponentDesignController;
		}

		public ConfigurationControllerEJBService getConfigurationController()
		{
			if(m_ConfigurationController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/ConfigurationControllerEJB";
				string serviceId = m_ClientController.getConfigurationControllerAsString();
				m_ConfigurationController = new ConfigurationControllerEJBService(serviceId,url);
			}
			return m_ConfigurationController;
		}

		public DefinitionControllerEJBService getDefinitionController()
		{
			if(m_DefinitionController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/DefinitionControllerEJB";
				string serviceId = m_ClientController.getDefinitionControllerAsString();
				m_DefinitionController = new DefinitionControllerEJBService(serviceId,url);
			}
			return m_DefinitionController;
		}

		public FilePlanComponentContentControllerEJBService getFilePlanComponentContentController()
		{
			if(m_FilePlanComponentContentController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/FilePlanComponentContentControllerEJB";
				string serviceId = m_ClientController.getFilePlanComponentContentControllerAsString();
				m_FilePlanComponentContentController = new FilePlanComponentContentControllerEJBService(serviceId,url);
			}
			return m_FilePlanComponentContentController;
		}

		public FilePlanComponentControllerEJBService getFilePlanComponentController()
		{
			if(m_FilePlanComponentController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/FilePlanComponentControllerEJB";
				string serviceId = m_ClientController.getFilePlanComponentControllerAsString();
				m_FilePlanComponentController = new FilePlanComponentControllerEJBService(serviceId,url);
			}
			return m_FilePlanComponentController;
		}

		public FilePlanDesignControllerEJBService getFilePlanDesignController()
		{
			if(m_FilePlanDesignController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/FilePlanDesignControllerEJB";
				string serviceId = m_ClientController.getFilePlanDesignControllerAsString();
				m_FilePlanDesignController = new FilePlanDesignControllerEJBService(serviceId,url);
			}
			return m_FilePlanDesignController;
		}

		public ImportExportControllerEJBService getImportExportController()
		{
			if(m_ImportExportController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/ImportExportControllerEJB";
				string serviceId = m_ClientController.getImportExportControllerAsString();
				m_ImportExportController = new ImportExportControllerEJBService(serviceId,url);
			}
			return m_ImportExportController;
		}

		public LifeCycleDesignControllerEJBService getLifeCycleDesignController()
		{
			if(m_LifeCycleDesignController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/LifeCycleDesignControllerEJB";
				string serviceId = m_ClientController.getLifeCycleDesignControllerAsString();
				m_LifeCycleDesignController = new LifeCycleDesignControllerEJBService(serviceId,url);
			}
			return m_LifeCycleDesignController;
		}

		public NavigationControllerEJBService getNavigationController()
		{
			if(m_NavigationController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/NavigationControllerEJB";
				string serviceId = m_ClientController.getNavigationControllerAsString();
				m_NavigationController = new NavigationControllerEJBService(serviceId,url);
			}
			return m_NavigationController;
		}

		public QueryControllerEJBService getQueryController()
		{
			if(m_QueryController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/QueryControllerEJB";
				string serviceId = m_ClientController.getQueryControllerAsString();
				m_QueryController = new QueryControllerEJBService(serviceId,url);
			}
			return m_QueryController;
		}

		public RecordsOperationsControllerEJBService getRecordsOperationsController()
		{
			if(m_RecordsOperationsController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/RecordsOperationsControllerEJB";
				string serviceId = m_ClientController.getRecordsOperationsControllerAsString();
				m_RecordsOperationsController = new RecordsOperationsControllerEJBService(serviceId,url);
			}
			return m_RecordsOperationsController;
		}

		public SecurityControllerEJBService getSecurityController()
		{
			if(m_SecurityController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/SecurityControllerEJB";
				string serviceId = m_ClientController.getSecurityControllerAsString();
				m_SecurityController = new SecurityControllerEJBService(serviceId,url);
			}
			return m_SecurityController;
		}

		public SecurityManagementControllerEJBService getSecurityManagementController()
		{
			if(m_SecurityManagementController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/SecurityManagementControllerEJB";
				string serviceId = m_ClientController.getSecurityManagementControllerAsString();
				m_SecurityManagementController = new SecurityManagementControllerEJBService(serviceId,url);
			}
			return m_SecurityManagementController;
		}

		public TemplateControllerEJBService getTemplateController()
		{
			if(m_TemplateController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/TemplateControllerEJB";
				string serviceId = m_ClientController.getTemplateControllerAsString();
				m_TemplateController = new TemplateControllerEJBService(serviceId,url);
			}
			return m_TemplateController;
		}

		public ToolsControllerEJBService getToolsController()
		{
			if(m_ToolsController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/ToolsControllerEJB";
				string serviceId = m_ClientController.getToolsControllerAsString();
				m_ToolsController = new ToolsControllerEJBService(serviceId,url);
			}
			return m_ToolsController;
		}

		public UserControllerEJBService getUserController()
		{
			if(m_UserController == null)
			{
				string url = "http://" + m_serverNamePort + "/IRMWebServices/services/UserControllerEJB";
				string serviceId = m_ClientController.getUserControllerAsString();
				m_UserController = new UserControllerEJBService(serviceId,url);
			}
			return m_UserController;
		}

	}
}
