using System;
using System.Xml;
using System.Collections;

namespace com.ibm.gre
{
	/// <summary>
	/// The IRMField class encapsulates a Field element from
	/// a Records manager Complex Fields Collection returned by one of
	/// the TemplateControllerEJB methods.
	/// The class contains methods for getting attribute values,
	/// getting and setting Field values and other utility functions.
	/// </summary>
	public class IRMField
	{
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="field">An XML Field element from a Records Manager Fields Collection</param>
		public IRMField(XmlElement field)
		{
			m_field = field;
		}

		/// <summary>
		/// Returns the constant value specifying the type of control to display.
		/// </summary>
		public IRMConstants.ProfileControlType ControlType //IRMConstants.ProfileControlType
		{
			get
			{
				string def = Convert.ToString((int)IRMConstants.ProfileControlType.SINGLELINE);
				return (IRMConstants.ProfileControlType)Convert.ToInt32(getVal("ControlType",def));
			}
		}

		/// <summary>
		/// Returns the FlPlnCmpntAttrDefColNm for this field.
		/// </summary>
		/// <returns>File Plan Component Attribute Column Name</returns>
		new public string ToString()
		{
			return FlPlnCmpntAttrDefColNm;
		}

		/// <summary>
		/// File Plan Component Attribute Column Name
		/// </summary>
		public string FlPlnCmpntAttrDefColNm
		{
			get
			{
				if(m_strFlPlnCmpntAttrDefColNm == null)
				{
					m_strFlPlnCmpntAttrDefColNm = m_field.Attributes.GetNamedItem("FlPlnCmpntAttrDefColNm").Value;
				}
				return m_strFlPlnCmpntAttrDefColNm;
			}
		}

		/// <summary>
		/// File Plan Component Definition ID
		/// </summary>
		public int FlPlnCmpntDefID
		{
			get
			{
				return Convert.ToInt32(getVal("FlPlnCmpntDefID","0"));
			}
		}

		/// <summary>
		/// File Plan Component Attribute Definition Data Type
		/// </summary>
		public IRMConstants.AttributeDataTypes FlPlnCmpntAttrDefDatTyp
		{
			get
			{
				string def = "" + IRMConstants.AttributeDataTypes.STRING;
				return (IRMConstants.AttributeDataTypes)Convert.ToInt32(getVal("FlPlnCmpntAttrDefDatTyp",def));
			}
		}

		/// <summary>
		/// Attribute Control Mask
		/// </summary>
		public int AttrCntrlMsk
		{
			get
			{
				return Convert.ToInt32(getVal("AttrCntrlMsk","0"));
			}
		}

		/// <summary>
		/// Caption
		/// </summary>
		public string Caption
		{
			get
			{
				if(m_strCaption == null)
				{
					m_strCaption = getVal("Caption","");
				}
				if("".Equals(m_strCaption))
				{
					m_strCaption = getVal("FlPlnCmpntAttrDefCapt", "");
				}
				return m_strCaption;
			}
		}

		/// <summary>
		/// Field Level Access
		/// </summary>
		public bool FldLvlAccess
		{
			get
			{
				return (getVal("FldLvlAccess","1") != "0");
			}
		}

		/// <summary>
		/// Mandatory flag.
		/// </summary>
		public bool Mandatory
		{
			get
			{
				return (getVal("Mandatory","0") != "0");
			}
		}

		/// <summary>
		/// Multi valued flag
		/// </summary>
		public bool MultiValuedFlg
		{
			get
			{
				return (getVal("MultiValuedFlg","0") != "0");
			}
		}

		/// <summary>
		/// Read Only flag
		/// </summary>
		public bool ReadOnly
		{
			get
			{
				return (getVal("ReadOnly","0") != "0");
			}
		}

		/// <summary>
		/// Returns true if this field contains a personal Pick List
		/// </summary>
		public bool IsPersonalPickList
		{
			get
			{
				return (getVal("IsPersonalPickList","0") != "0");
			}
		}

		/// <summary>
		/// Data Type Maximum Length
		/// </summary>
		public int DatTypMaxLngth
		{
			get
			{
				return Convert.ToInt32(getVal("DatTypMaxLngth","0"));
			}
		}

		/// <summary>
		/// Pick List ID
		/// </summary>
		public int PckLstID
		{
			get
			{
				return Convert.ToInt32(getVal("PckLstID","0"));
			}
		}

		/// <summary>
		/// Returns the XML Field element with all attributes removed
		/// except the FlPlnCmpntAttrDefColNm attribute.
		/// </summary>
		/// <returns>The XML Field element.</returns>
		public XmlElement getFieldForAdd()
		{
			XmlElement field = (XmlElement)m_field.Clone();
			//Remove Extraneous Attributes
			XmlAttributeCollection attrs = field.Attributes;
			for(int i = attrs.Count - 1; i > -1; i--)
			{
				if(attrs.Item(i).Name != "FlPlnCmpntAttrDefColNm")
				{
					field.RemoveAttribute(attrs.Item(i).Name);
				}
			}
			return field;
		}

		/// <summary>
		/// The value for the field.
		/// </summary>
		public string Value 
		{
			get
			{
				//If the field is a Virtual Field return inner XML
				if(IRMAttrCntrlMsk.isVirtual(AttrCntrlMsk))
				{
					return m_field.InnerXml;
				}
				if(MultiValuedFlg)
				{
					return m_field.InnerXml;
				}
				return m_field.InnerText;
			}
			set
			{
				if(IRMAttrCntrlMsk.isVirtual(AttrCntrlMsk))
				{
					m_field.InnerXml = value;
					return;
				}
				if(MultiValuedFlg)
				{
					m_field.InnerXml = value;
					return;
				}
				m_field.InnerText = value;
			}
		}

		/// <summary>
		/// Specify if the field should be visible on a form
		/// </summary>
		public bool isVisible
		{
			get
			{
				return m_bVisible;
			}
			set
			{
				m_bVisible = value;
			}
		}

		/// <summary>
		/// Specifies if the field should be used
		/// </summary>
		public bool IsActive
		{
			get
			{
				return m_bIsActive;
			}
			set
			{
				m_bIsActive = value;
			}
		}

		/// <summary>
		/// Gets the value for a specified attribute
		/// </summary>
		/// <param name="attrName">The name of the attribute to return th value for</param>
		/// <param name="defaultVal">The default value to use if the attribute does not exist or has no value.</param>
		/// <returns></returns>
		private string getVal(string attrName, string defaultVal)
		{
			XmlNode node = m_field.Attributes.GetNamedItem(attrName);
			if(node != null)
			{
				string temp = node.Value;
				if(temp.Length > 0)
				{
					defaultVal = temp;
				}
			}
			return defaultVal;
		}

		private XmlElement m_field;
		//These fields are usually used many times so save them
		private string m_strCaption = null;
		private string m_strFlPlnCmpntAttrDefColNm = null;
		private bool m_bVisible = true;
		private bool m_bIsActive = true;
	}

	/// <summary>
	/// The IRMFields class encapsulates a Fields element from
	/// a Records manager Complex Fields Collection returned by one of
	/// the TemplateControllerEJB methods.
	/// This class can be used to help get and set values for
	/// Adding, Editing and Displaying Records Manager Components.
	/// </summary>
	/// <remarks>
	/// See the documentation for the TemplateControllerEJB interface
	/// contained in the com.ibm.gre.engine.ejb.facade package in the
	/// IBM DB2 Records Manager API Reference for examples and explanations
	/// of methods that will return or accept Complex Fields Collection XML
	/// values. 
	/// </remarks>
	public class IRMFields
	{
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="xml">A Complex Fields Collection String value.
		/// </param>
		public IRMFields(string xml)
		{
			m_fields = new ArrayList();
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			m_root = doc.DocumentElement;
			string xpath = "./Field";
			XmlNodeList fields = m_root.SelectNodes(xpath);
			for(int i = 0; i < fields.Count; i++)
			{
				IRMField irm = new IRMField((XmlElement)fields.Item(i));
				
				//Get the File Plan Component Definition ID from one of the fields
				if(m_nFlPlnCmpntDefID == 0)
				{
					m_nFlPlnCmpntDefID = irm.FlPlnCmpntDefID;
				}
				
				//Always set the visability of RowVrsn to false;
				//User can re-set to true if they wish to see this
				if(irm.FlPlnCmpntAttrDefColNm == "RowVrsn")
				{
					irm.isVisible = false;
				}
				if(irm.FlPlnCmpntAttrDefColNm == "Profile_Options")
				{
					m_Profile_Options = irm;
				}
				else if(irm.FlPlnCmpntAttrDefColNm == "Default_Options")
				{
					m_Default_Options = irm;
				}
				else
				{
					m_fields.Add(irm);
				}
			}
		}

		/// <summary>
		/// The ID of the Component Definition type that the Fields
		/// class reprepresents. 
		/// </summary>
		public int FlPlnCmpntDefID
		{
			get
			{
				return m_nFlPlnCmpntDefID;
			}
		}

		/// <summary>
		/// The number of Field objects that the
		/// class contains.
		/// </summary>
		public int FieldsCount
		{
			get
			{
				return m_fields.Count;
			}
		}

		/// <summary>
		/// The number of Field objects that have
		/// been specified to be displayed.
		/// </summary>
		public int DisplayCount
		{
			get
			{
				int count = 0;
				for(int i = 0; i < m_fields.Count; i++)
				{
					IRMField field = (IRMField)m_fields[i];
					if(field.isVisible) count++;
				}
				return count;
			}
		}

		/// <summary>
		/// Get the IRMField object for the specified index.
		/// </summary>
		/// <param name="index">The integer value from 0 to n indicating the IRMField object to return.</param>
		/// <returns>An IRMField object containing attributes and values.</returns>
		public IRMField getField(int index)
		{
			return (IRMField)m_fields[index];
		}

		/// <summary>
		/// Gets the IRMField object for the specified FlPlnCmpntAttrDefColNm. 
		/// </summary>
		/// <param name="strFlPlnCmpntAttrDefColNm">File Plan Attribute Definition Column Name.</param>
		/// <returns>An IRMField object containing attributes and values.</returns>
		public IRMField getFieldByFlPlnCmpntAttrDefColNm(string strFlPlnCmpntAttrDefColNm)
		{
			for(int i = 0; i < m_fields.Count; i++)
			{
				IRMField field = (IRMField)m_fields[i];
				if(strFlPlnCmpntAttrDefColNm == field.FlPlnCmpntAttrDefColNm)
				{
					return field;			
				}
			}
			throw new Exception("Could not find Field: " + strFlPlnCmpntAttrDefColNm);
		}

		/// <summary>
		/// Set the value of the underlying Field Element for the Component
		/// Attribute specified by the FlPlnCmpntAttrDefColNm.
		/// </summary>
		/// <param name="strFlPlnCmpntAttrDefColNm">File Plan Attribute Definition Column Name.</param>
		/// <param name="val">The value to assign to the specified Field.</param>
		/// <returns>Returns true if the value was successfully set.</returns>
		public bool setFieldByFlPlnCmpntAttrDefColNm(string strFlPlnCmpntAttrDefColNm, string val)
		{
			for(int i = 0; i < m_fields.Count; i++)
			{
				IRMField field = (IRMField)m_fields[i];
				if(strFlPlnCmpntAttrDefColNm == field.FlPlnCmpntAttrDefColNm)
				{
					field.Value = val;
					return true; //TODO add validation here			
				}
			}
			throw new Exception("Could not find Field: " + strFlPlnCmpntAttrDefColNm);
		}

		/// <summary>
		/// Set the visability of a specified Field.
		/// </summary>
		/// <param name="strFlPlnCmpntAttrDefColNm">File Plan Attribute Definition Column Name.</param>
		/// <param name="makeVisable">Setting this value to false indicates the field should not be displayed.</param>
		public void setVisabilityByFlPlnCmpntAttrDefColNm(string strFlPlnCmpntAttrDefColNm, bool makeVisable)
		{
			for(int i = 0; i < m_fields.Count; i++)
			{
				IRMField field = (IRMField)m_fields[i];
				if(strFlPlnCmpntAttrDefColNm == field.FlPlnCmpntAttrDefColNm)
				{
					field.isVisible = makeVisable;
				}
			}
		}

		/// <summary>
		/// This is a convenience method for setting the visability
		/// attribute to Security Fields, indicating they should
		/// not be displayed. Host applications that do not use Security
		/// Fields can use this method.
		/// </summary>
		public void hideSecurityFields()
		{
			setVisabilityByFlPlnCmpntAttrDefColNm("SecClsID",false);
			setVisabilityByFlPlnCmpntAttrDefColNm("SecClsCommonOwnerID",false);
			setVisabilityByFlPlnCmpntAttrDefColNm("SecClsInfo",false);
		}

		/// <summary>
		/// Sets a flag indicating whether a specified field should be include
		/// in values for Add or update.
		/// </summary>
		/// <param name="strFlPlnCmpntAttrDefColNm">File Plan Attribute Definition Column Name.</param>
		/// <param name="active">Set this value to false to indicate the field should not be used.</param>
		public void setActiveStateByFlPlnCmpntAttrDefColNm(string strFlPlnCmpntAttrDefColNm, bool active)
		{
			for(int i = 0; i < m_fields.Count; i++)
			{
				IRMField field = (IRMField)m_fields[i];
				if(strFlPlnCmpntAttrDefColNm == field.FlPlnCmpntAttrDefColNm)
				{
					field.IsActive = active;
				}
			}
		}

		/// <summary>
		/// A convenience method for in-activating security fields.
		/// Host applications that do not use security can use this method.
		/// </summary>
		public void inactivateSecurityFields()
		{
			setActiveStateByFlPlnCmpntAttrDefColNm("SecClsID",false);
			setActiveStateByFlPlnCmpntAttrDefColNm("SecClsCommonOwnerID",false);
			setActiveStateByFlPlnCmpntAttrDefColNm("SecClsInfo",false);
		}

		/// <summary>
		/// This method returns a properly formatted XML
		/// string that can be used as input to calls to the
		/// Records Manager TemplateControllerEJB methods
		/// addNewFilePlanComponent or addNewSystemComponent.
		/// Can also be used as input to the Records Manager
		/// Component Update methods.
		/// </summary>
		/// <returns>Correctly formatted XML string.</returns>
		public string getXMLForAdd()
		{
			XmlDocument doc = new XmlDocument();
			doc.LoadXml("<Fields/>");
			XmlElement fields = doc.DocumentElement;
			for(int i = 0; i < m_fields.Count; i++)
			{
				IRMField field = (IRMField)m_fields[i];
				if(field.IsActive)
				{
					fields.AppendChild(doc.ImportNode(field.getFieldForAdd(),true));
				}
			}
			return fields.OuterXml;
		}

		/// <summary>
		/// This method returns the number of characters
		/// contained in the longest Caption in the Fields
		/// Collection.
		/// </summary>
		public string MaxCaption
		{
			get
			{
				if(m_nMaxCaptionLength == 0)
				{
					for(int i = 0; i < m_fields.Count; i++)
					{
						IRMField field = (IRMField)m_fields[i];
						int len = field.Caption.Length; 
						if(len > m_nMaxCaptionLength)
						{
							m_nMaxCaptionLength = len;
							m_nMaxCaptionIndex = i;
						}
					}
				}
				return ((IRMField)m_fields[m_nMaxCaptionIndex]).Caption;
			}
		}

		//Private Member Variables
		private XmlElement m_root;
		private ArrayList m_fields;
		private int m_nMaxCaptionLength = 0;
		private int m_nMaxCaptionIndex = 0;
		private int m_nFlPlnCmpntDefID = 0;

		//Profile and Default data
		IRMField m_Profile_Options = null;
		IRMField m_Default_Options = null;

	}
}
