using System;
using System.Xml;
using System.Collections;

namespace com.ibm.gre
{
	/// <summary>
	/// The Column Class encapsulate the standard attribute values
	/// for each Item element contained in the MetaData element
	/// from a Records Manager Query operation.
	/// These values describe each column that is returned for the query.
	/// See the documentation of the runQuery method in the QueryControllerEJB
	/// class for more information on the XML structure. 
	/// </summary>
	public class Column
	{
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="node">
		/// This is an XML Item element from the MetaData section
		/// of a query result string.
		/// </param>
		public Column(XmlNode node)
		{
			m_strFlPlnCmpntAttrDefCapt = node.Attributes.GetNamedItem("FlPlnCmpntAttrDefCapt").Value;
			m_strFlPlnCmpntAttrDefColNm = node.Attributes.GetNamedItem("FlPlnCmpntAttrDefColNm").Value;
			string temp = node.Attributes.GetNamedItem("FlPlnCmpntAttrDefDatTyp").Value;
			m_nFlPlnCmpntAttrDefDatTyp = Convert.ToByte(temp);
			XmlNode testNode = node.Attributes.GetNamedItem("FlPlnCmpntAttrDefNo");
			if(testNode == null)
			{
				m_nFlPlnCmpntAttrDefNo = -999; //Indicates the column does not have a File Plan Component Attribute Definition Number
			}
			else
			{
				temp = testNode.Value;
				m_nFlPlnCmpntAttrDefNo = temp == "" ? -999 : Convert.ToInt32(temp);   
			}
			m_nFlPlnCmpntDefID = Convert.ToInt32(node.Attributes.GetNamedItem("FlPlnCmpntDefID").Value);
			testNode = node.Attributes.GetNamedItem("PckLstID");
			if(testNode == null)
			{
				m_nPckLstID = 0; //Indicates the column does not have a Pick List
			}
			else
			{
				temp = testNode.Value;
				m_nPckLstID = temp == "" ? 0 : Convert.ToInt32(temp);   
			}
			//New for version 4.1.3
			testNode = node.Attributes.GetNamedItem("Visible");
			if(testNode == null)
			{
				m_isVisible = true; //TODO check if this condition is possible
			}
			else
			{
				temp = testNode.Value;
				m_isVisible = !temp.Equals("0") ? true : false;   
			}

		}

		/// <summary>
		/// Returns true if the Visible attribute did not have a value of 0
		/// </summary>
		public bool IsVisible
		{
			get
			{
				return m_isVisible;
			}
		}

		/// <summary>
		/// The suggested percentage of the total width for all Columns
		/// that this column should take up.
		/// </summary>
		public int PercentWidth
		{
			get
			{
				return m_nPercentWidth;
			}
			set
			{
				m_nPercentWidth = value;
			}
		}

		/// <summary>
		/// The Caption that should be displayed for this column.
		/// </summary>
		public string FlPlnCmpntAttrDefCapt
		{
			get
			{
				return m_strFlPlnCmpntAttrDefCapt;
			}
		}

		/// <summary>
		/// The unique name for the Component Attribute represented by the column.
		/// </summary>
		public string FlPlnCmpntAttrDefColNm
		{
			get
			{
				return m_strFlPlnCmpntAttrDefColNm;
			}
		}

		/// <summary>
		/// The Records Manager data type for the column.
		/// </summary>
		public byte FlPlnCmpntAttrDefDatTyp
		{
			get
			{
				return m_nFlPlnCmpntAttrDefDatTyp;
			}
		}

		/// <summary>
		/// File Plan Component Attribute Definition Number
		/// </summary>
		public int FlPlnCmpntAttrDefNo
		{
			get
			{
				return m_nFlPlnCmpntAttrDefNo;
			}
		}
		
		/// <summary>
		/// The Component Definition Id of the Component 
		/// being reported on.
		/// </summary>
		public int FlPlnCmpntDefID
		{
			get
			{
				return m_nFlPlnCmpntDefID;
			}
		}
		
		/// <summary>
		/// The Id of a PickList that values for
		/// this column can be selected from.
		/// </summary>
		public int PckLstID
		{
			get
			{
				return m_nPckLstID;
			}
		}

		private string m_strFlPlnCmpntAttrDefCapt;
		private string m_strFlPlnCmpntAttrDefColNm;
		private byte m_nFlPlnCmpntAttrDefDatTyp;
		private int m_nFlPlnCmpntAttrDefNo;
		private int m_nFlPlnCmpntDefID;
		private int m_nPckLstID;
		private int m_nPercentWidth = 10; //NOTE the UI developer must set this value the default will not be realistic.
		//New for version 4.1.3
		private bool m_isVisible;
	}

	/// <summary>
	/// The IRMResultList encapsulates the XML returned from
	/// a Records Manager Query.
	/// The class contains methods for iterating through
	/// all the results from a query while
	/// iterating through all the individual Columns
	/// for each result.
	/// Tables of values can be built using this process.
	/// There are methods that can be used to limit the number
	/// of columns to display. 
	/// </summary>
	public class IRMResultList
	{
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="irmQuery">
		/// An IRMQuery object.
		/// This constructor is provide for future
		/// expansion of the class so that it may be possible
		/// to re-order column results for
		/// stateless clients.
		/// </param>
		public IRMResultList(IRMQuery irmQuery)
		{
			m_SelectedColumns = new ArrayList();
			m_irmQuery = irmQuery;
			m_doc = new XmlDocument();
			reLoad();
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="resultList">
		/// The XML formatted string returned from
		/// the QueryControllerEJB.runQuery method.
		/// NOTE the queryResultOption value supplied
		/// to the the runQuery method must be the value
		/// com.ibm.gre.engine.util.QueryResultOptions.INCLUDE_COLUMNS
		/// so that the MetaData element is included
		/// in the result. 
		/// </param>
		public IRMResultList(string resultList)
		{
			m_SelectedColumns = new ArrayList();
			m_irmQuery = null;
			m_doc = new XmlDocument();
			m_doc.LoadXml(resultList);
			XmlElement root = m_doc.DocumentElement;

			//XmlNodeList nodes = root.GetElementsByTagName("MetaData");
			//m_MetaData = (XmlElement)nodes[0];
			m_MetaData = (XmlElement)root.SelectSingleNode("MetaData");

			//nodes = root.GetElementsByTagName("Data");
			//m_Data = (XmlElement)nodes[0];
			m_Data = (XmlElement)root.SelectSingleNode("Data");

			
			m_Items = m_Data.SelectNodes("./List/Item");
		}

		/// <summary>
		/// This method specifies the name of a column
		/// to include in the displayed results.
		/// If no names are specified the class will
		/// include all the columns specified in the
		/// XML used to construct this object.
		/// </summary>
		/// <param name="strFlPlnCmpntAttrDefColNm">
		/// The File Plan Component Attribute Column Name
		/// that should be displayed.
		/// </param>
		public void AddColumn(string strFlPlnCmpntAttrDefColNm)
		{
			bool ok = false;
			XmlNodeList items = m_MetaData.SelectNodes("./List/Item");
			for(int i = 0; i < items.Count; i++)
			{
				string test = items.Item(i).Attributes.GetNamedItem("FlPlnCmpntAttrDefColNm").Value;
				if(strFlPlnCmpntAttrDefColNm == test)
				{
					Column col = new Column(items.Item(i));
					m_SelectedColumns.Add(col);
					ok = true;
					break;
				}
			}
			if(!ok)
			{
				throw new Exception("The FlPlnCmpntAttrDefColNm: " + strFlPlnCmpntAttrDefColNm + " does not exist in the Result List MetaData");
			}
		}

		/// <summary>
		/// The number of columns to be displayed.
		/// Note this is either the number of columns
		/// that are specified using the above AddColumn
		/// method or all of the columns in the original
		/// XML if the AddColumn method is never called. 
		/// </summary>
		public int ColumnCount
		{
			get
			{
				makeColumnsArray();
				return m_SelectedColumns.Count;
			}
		}

		/// <summary>
		/// Gets the Caption of a table column
		/// for the specified index.
		/// </summary>
		/// <param name="index">The index into the collection
		/// of Columns for this result list.</param>
		/// <returns>The Caption to display in the table header.</returns>
		public string getFlPlnCmpntAttrDefCapt(int index)
		{
			makeColumnsArray();
			return ((Column)m_SelectedColumns[index]).FlPlnCmpntAttrDefCapt;
		}

		/// <summary>
		/// Gets the FlPlnCmpntAttrDefColNm of a table
		/// column for the specified index.
		/// This value is used in the getItemByFlPlnCmpntAttrDefColNm
		/// method to specify which value to return.
		/// </summary>
		/// <param name="index">The index into the collection
		/// of Columns for this result list.</param>
		/// <returns>The FlPlnCmpntAttrDefColNm to use to get
		/// the values for the column value.</returns>
		public string getFlPlnCmpntAttrDefColNm(int index)
		{
			makeColumnsArray();
			return ((Column)m_SelectedColumns[index]).FlPlnCmpntAttrDefColNm;
		}

		/// <summary>
		/// The suggested percentage of the total width for all Columns
		/// that the specified column should take up.
		/// </summary>
		/// <param name="index">The index into the collection
		/// of Columns for this result list.</param>
		/// <returns>Percentage Width</returns>
		public int getPercentageColumnWidth(int index)
		{
			makeColumnWidths();
			return ((Column)m_SelectedColumns[index]).PercentWidth;
		}

		/// <summary>
		/// The Number of rows that will be returned in the result set.
		/// </summary>
		public int ItemCount
		{
			get
			{
				return m_Items.Count;
			}
		}

		/// <summary>
		/// This method gets the value for the column as specified
		/// by the strFlPlnCmpntAttrDefColNm param for the result set
		/// row specified by the index parameter.
		/// You fill table values by iterating through all
		/// index values from 0 to ItemCount.
		/// For each row you then iterate through all columns
		/// and call the getFlPlnCmpntAttrDefColNm method
		/// for each column value from 0 to ColumnCount to get
		/// the value to supply for the strFlPlnCmpntAttrDefColNm param.
		/// </summary>
		/// <param name="index">Use this value to specify the index value of
		/// the Item (row) in the result set to return the value for.</param>
		/// <param name="strFlPlnCmpntAttrDefColNm">Use this value to specify the
		/// name of the column to return the result for.</param>
		/// <returns>Returns the value of the named XML attribute in the specified position.</returns>
		public string getItemByFlPlnCmpntAttrDefColNm(int index, string strFlPlnCmpntAttrDefColNm)
		{
			string test = m_Items.Item(index).OuterXml;
			XmlAttribute attr = (XmlAttribute)m_Items.Item(index).Attributes.GetNamedItem(strFlPlnCmpntAttrDefColNm);
			if(attr == null)
			{
				return "";
			}
			return attr.Value;
		}

		/// <summary>
		/// This is a variation on the above method.
		/// The FlPlnCmpntAttrDefColNm does not have to be known
		/// only the index of the column in the table.
		/// </summary>
		/// <param name="row">Use this value to specify the index value of
		/// the Item (row) in the result set to return the value for.</param>
		/// <param name="column">Use this value to specify the index value of the column
		/// in the table</param>
		/// <returns>Returns the value of the XML attribute in the specified position.</returns>
		public string getItemByRowColumnIndex(int row, int column)
		{
			//string test = m_Items.Item(row).OuterXml;
			Column col = (Column)m_SelectedColumns[column];
			XmlAttribute attr = (XmlAttribute)m_Items.Item(row).Attributes.GetNamedItem(col.FlPlnCmpntAttrDefColNm);
			if(attr == null)
			{
				return "";
			}
			return attr.Value;
		}

		/// <summary>
		/// Set the wheight value that is used for
		/// BINARY and BOOLEAN values when the
		/// percentage width is calculated for these types.
		/// The default value is 1.
		/// </summary>
		public byte Wheight1
		{
			set
			{
				weight1 = value;
			}
		}

		/// <summary>
		/// Set the wheight value that is used for
		/// DATE, DOUBLE, INTEGER and VIRTUAL values when the
		/// percentage width is calculated for these types.
		/// The default value is 2.
		/// This means a DATE value will take up twice as
		/// much column width as a BOOLEAN
		/// </summary>
		public byte Wheight2
		{
			set
			{
				weight2 = value;
			}
		}

		/// <summary>
		/// Set the wheight value that is used for
		/// DATETIME, CLOB and STRING values when the
		/// percentage width is calculated for these types.
		/// The default value is 3.
		/// This means a DATE value will take up three times as
		/// much column width as a BOOLEAN
		/// </summary>
		public byte Wheight3
		{
			set
			{
				weight3 = value;
			}
		}

		private void reLoad()
		{
			m_doc.LoadXml(m_irmQuery.ResultList);
			XmlElement root = m_doc.DocumentElement;
			m_MetaData = (XmlElement)root.SelectSingleNode("MetaData");
			m_Data = (XmlElement)root.SelectSingleNode("Data");
			m_Items = m_Data.SelectNodes("./List/Item");
		}

		private void makeColumnsArray()
		{
			//If the developer has not specified a set of columns then
			//use all the columns available from the MetaData
			if(m_SelectedColumns.Count == 0)
			{
				XmlNodeList items = m_MetaData.SelectNodes("./List/Item");
				for(int i = 0; i < items.Count; i++)
				{
					Column col = new Column(items.Item(i));
					if(col.IsVisible)
					{
						m_SelectedColumns.Add(col);
					}
				}
			}
		}

		//This is a convenience method to determine the percentage widths of the displayed columns
		//Using the specified wheightings the defaults being bool = 1, numeric = 2, string = 3
		private void makeColumnWidths()
		{
			if(!widthsDone)
			{
				float totalWeights = 0;
				widthsDone = true;
				for(int i = 0; i < m_SelectedColumns.Count; i++)
				{
					totalWeights += wheightByFlPlnCmpntAttrDefDatTyp(((Column)m_SelectedColumns[i]).FlPlnCmpntAttrDefDatTyp);	
				}
				for(int i = 0; i < m_SelectedColumns.Count; i++)
				{
					float weight = wheightByFlPlnCmpntAttrDefDatTyp(((Column)m_SelectedColumns[i]).FlPlnCmpntAttrDefDatTyp);
					int percentWidth =  (int)((weight/totalWeights) * 100);
					((Column)m_SelectedColumns[i]).PercentWidth = percentWidth;
				}
			}
		}

		private int wheightByFlPlnCmpntAttrDefDatTyp(int nFlPlnCmpntAttrDefDatTyp)
		{
			switch((IRMConstants.AttributeDataTypes)nFlPlnCmpntAttrDefDatTyp)
			{
				case IRMConstants.AttributeDataTypes.BINARY:
					return weight1;
				case IRMConstants.AttributeDataTypes.BOOLEAN:
					return weight1;
				case IRMConstants.AttributeDataTypes.CLOB:
					return weight3;
				case IRMConstants.AttributeDataTypes.DATE:
					return weight2;
				case IRMConstants.AttributeDataTypes.DATETIME:
					return weight3;
				case IRMConstants.AttributeDataTypes.DOUBLE:
					return weight2;
				case IRMConstants.AttributeDataTypes.INTEGER:
					return weight2;
				case IRMConstants.AttributeDataTypes.STRING:
					return weight3;
				case IRMConstants.AttributeDataTypes.VIRTUAL:
					return weight2;
				default:
					return weight2;
			}
		}

		protected XmlNodeList m_Items;

		private IRMQuery m_irmQuery;
		private XmlDocument m_doc;
		private XmlElement m_MetaData;
		private XmlElement m_Data;
		private ArrayList m_SelectedColumns;
		private bool widthsDone = false;
		private byte weight1 = 1;
		private byte weight2 = 2;
		private byte weight3 = 3;
	}
}
